package cy.agorise.graphenej;

import com.google.common.primitives.UnsignedLong;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;

import org.junit.Assert;
import org.junit.Test;

import java.lang.reflect.Type;
import java.util.List;

import cy.agorise.graphenej.models.WitnessResponse;

/**
 * Testing the {@link OrderBook} class
 */

public class OrderBookTest {

    @Test
    public void testRequiredBase(){
        String serializedOrderBook = "{\"id\": 1,\"result\": [{\"id\": \"1.7.37284933\",\"expiration\": \"2018-11-17T00:03:49\",\"seller\": \"1.2.132823\",\"for_sale\": 10,\"sell_price\": {\"base\": {\"amount\": 1,\"asset_id\": \"1.3.121\"},\"quote\": {\"amount\": 10,\"asset_id\": \"1.3.0\"}},\"deferred_fee\": 0},{\"id\": \"1.7.37284933\",\"expiration\": \"2018-11-17T00:03:49\",\"seller\": \"1.2.132823\",\"for_sale\": 20,\"sell_price\": {\"base\": {\"amount\": 1,\"asset_id\": \"1.3.121\"},\"quote\": {\"amount\": 20,\"asset_id\": \"1.3.0\"}},\"deferred_fee\": 0}]}";
        GsonBuilder builder = new GsonBuilder();
        builder.registerTypeAdapter(AssetAmount.class, new AssetAmount.AssetAmountDeserializer());
        builder.registerTypeAdapter(UserAccount.class, new UserAccount.UserAccountSimpleDeserializer());
        builder.registerTypeAdapter(LimitOrder.class, new LimitOrder.LimitOrderDeserializer());
        Type GetLimitOrdersResponse = new TypeToken<WitnessResponse<List<LimitOrder>>>() {}.getType();
        WitnessResponse<List<LimitOrder>> witnessResponse = builder.create().fromJson(serializedOrderBook, GetLimitOrdersResponse);
        List<LimitOrder> orders = witnessResponse.result;
        OrderBook orderBook = new OrderBook(orders);


        final Asset _quote = new Asset("1.3.121", "USD", 4);
        final Asset _base = new Asset("1.3.0", "BTS", 5);
        long _totalQuote = 14;
        UnsignedLong _totalBase = orderBook.calculateRequiredBase(new AssetAmount(UnsignedLong.valueOf(_totalQuote), _quote));
        Assert.assertEquals("Should buy 10 at 10 and 4 at 20, which sums up to 180",180, _totalBase.longValue());
        System.out.println(String.format("Base: %s, Quote: %s", _base.getObjectId(), _quote.getObjectId()));
        System.out.println(String.format("_totalQuote: %d, _totalBase: %d", _totalQuote, _totalBase.longValue()));
    }

    @Test
    public void testRequiredQuote(){
        String serializedOrderBook = "{\"id\":1,\"result\":[{\"id\":\"1.7.37284933\",\"expiration\":\"2018-11-17T00:03:49\",\"seller\":\"1.2.132823\",\"for_sale\":20,\"sell_price\":{\"base\":{\"amount\":20,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":0},{\"id\":\"1.7.37284933\",\"expiration\":\"2018-11-17T00:03:49\",\"seller\":\"1.2.132823\",\"for_sale\":100,\"sell_price\":{\"base\":{\"amount\":10,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":0}]}";
        GsonBuilder builder = new GsonBuilder();
        builder.registerTypeAdapter(AssetAmount.class, new AssetAmount.AssetAmountDeserializer());
        builder.registerTypeAdapter(UserAccount.class, new UserAccount.UserAccountSimpleDeserializer());
        builder.registerTypeAdapter(LimitOrder.class, new LimitOrder.LimitOrderDeserializer());
        Type GetLimitOrdersResponse = new TypeToken<WitnessResponse<List<LimitOrder>>>() {}.getType();
        WitnessResponse<List<LimitOrder>> witnessResponse = builder.create().fromJson(serializedOrderBook, GetLimitOrdersResponse);
        List<LimitOrder> orders = witnessResponse.result;
        OrderBook orderBook = new OrderBook(orders);


        final Asset _base = new Asset("1.3.121", "USD", 4);
        final Asset _quote = new Asset("1.3.0", "BTS", 5);
        long _totalBase = 2;
        UnsignedLong _totalQuote = orderBook.calculateObtainedQuote(new AssetAmount(UnsignedLong.valueOf(_totalBase), _base));
        Assert.assertEquals("Should be able to buy 20 on the first order and 10 on the second, totalling 30",30, _totalQuote.longValue());
        System.out.println(String.format("Base: %s, Quote: %s", _base.getObjectId(), _quote.getObjectId()));
        System.out.println(String.format("_totalQuote: %d, _totalBase: %d", _totalQuote.longValue(), _totalQuote.longValue()));
    }

    @Test
    public void testRequiredQuoteReal(){
        String serializedOrderBook = "{\"id\":1,\"result\":[{\"id\":\"1.7.39826107\",\"expiration\":\"2022-11-29T04:26:23\",\"seller\":\"1.2.384119\",\"for_sale\":100000,\"sell_price\":{\"base\":{\"amount\":100000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":5930000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826091\",\"expiration\":\"2022-11-29T04:26:06\",\"seller\":\"1.2.384119\",\"for_sale\":100000,\"sell_price\":{\"base\":{\"amount\":100000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":5950000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39825780\",\"expiration\":\"2022-11-29T04:24:56\",\"seller\":\"1.2.384119\",\"for_sale\":100000,\"sell_price\":{\"base\":{\"amount\":100000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":5970000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39827059\",\"expiration\":\"2017-11-29T05:31:34\",\"seller\":\"1.2.359723\",\"for_sale\":357277,\"sell_price\":{\"base\":{\"amount\":357277,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":21355520,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39825610\",\"expiration\":\"2022-11-29T04:23:34\",\"seller\":\"1.2.384119\",\"for_sale\":100000,\"sell_price\":{\"base\":{\"amount\":100000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":5990000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826428\",\"expiration\":\"2022-11-29T04:28:45\",\"seller\":\"1.2.384119\",\"for_sale\":1000000,\"sell_price\":{\"base\":{\"amount\":1000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":59990000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39825045\",\"expiration\":\"2022-11-29T04:20:49\",\"seller\":\"1.2.7003\",\"for_sale\":35754092,\"sell_price\":{\"base\":{\"amount\":51008622,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":3060517320,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":0},{\"id\":\"1.7.39827034\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.398049\",\"for_sale\":226328,\"sell_price\":{\"base\":{\"amount\":226328,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":13635626,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826622\",\"expiration\":\"2022-11-29T04:30:07\",\"seller\":\"1.2.384119\",\"for_sale\":1000000,\"sell_price\":{\"base\":{\"amount\":1000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":61000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39825800\",\"expiration\":\"1963-11-25T22:31:44\",\"seller\":\"1.2.95758\",\"for_sale\":1515000,\"sell_price\":{\"base\":{\"amount\":1515000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":92750445,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39827058\",\"expiration\":\"1963-11-29T12:38:45\",\"seller\":\"1.2.36449\",\"for_sale\":75750000,\"sell_price\":{\"base\":{\"amount\":75750000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":\"4638289114\",\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826511\",\"expiration\":\"2017-12-06T04:29:04\",\"seller\":\"1.2.376473\",\"for_sale\":1516515,\"sell_price\":{\"base\":{\"amount\":1516515,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":92858549,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39822796\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.165710\",\"for_sale\":151500,\"sell_price\":{\"base\":{\"amount\":151500,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":9285855,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39791452\",\"expiration\":\"2022-11-29T01:16:22\",\"seller\":\"1.2.28914\",\"for_sale\":16200000,\"sell_price\":{\"base\":{\"amount\":16200000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":1000000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826697\",\"expiration\":\"2022-11-29T04:30:22\",\"seller\":\"1.2.384119\",\"for_sale\":1000000,\"sell_price\":{\"base\":{\"amount\":1000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":62000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39767436\",\"expiration\":\"2022-11-28T23:16:04\",\"seller\":\"1.2.408360\",\"for_sale\":17934080,\"sell_price\":{\"base\":{\"amount\":17934080,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":1112535980,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39770442\",\"expiration\":\"2022-11-28T23:27:12\",\"seller\":\"1.2.429094\",\"for_sale\":1052917,\"sell_price\":{\"base\":{\"amount\":1052917,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":66221194,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1227},{\"id\":\"1.7.39787848\",\"expiration\":\"2022-11-29T00:57:45\",\"seller\":\"1.2.408360\",\"for_sale\":19945106,\"sell_price\":{\"base\":{\"amount\":19945106,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":1256447949,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1257},{\"id\":\"1.7.39826299\",\"expiration\":\"2022-11-29T04:27:44\",\"seller\":\"1.2.384119\",\"for_sale\":3000000,\"sell_price\":{\"base\":{\"amount\":3000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":189000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39726192\",\"expiration\":\"2022-11-28T17:25:39\",\"seller\":\"1.2.398578\",\"for_sale\":316200,\"sell_price\":{\"base\":{\"amount\":316200,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":20000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39724878\",\"expiration\":\"2022-11-28T17:15:53\",\"seller\":\"1.2.472756\",\"for_sale\":5987987,\"sell_price\":{\"base\":{\"amount\":5987987,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":382024828,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39762644\",\"expiration\":\"2022-11-28T23:00:35\",\"seller\":\"1.2.465467\",\"for_sale\":1252843,\"sell_price\":{\"base\":{\"amount\":1252843,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":80000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39817965\",\"expiration\":\"2022-11-29T03:45:43\",\"seller\":\"1.2.384119\",\"for_sale\":3000000,\"sell_price\":{\"base\":{\"amount\":3000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":192000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39718037\",\"expiration\":\"2018-11-28T16:10:58\",\"seller\":\"1.2.132823\",\"for_sale\":1052528,\"sell_price\":{\"base\":{\"amount\":1052528,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":67641517,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39743004\",\"expiration\":\"2022-11-28T20:28:12\",\"seller\":\"1.2.182034\",\"for_sale\":1547001,\"sell_price\":{\"base\":{\"amount\":1547001,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":100000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39718531\",\"expiration\":\"2022-11-28T16:17:33\",\"seller\":\"1.2.424868\",\"for_sale\":100000000,\"sell_price\":{\"base\":{\"amount\":100000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":\"6464124111\",\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39715891\",\"expiration\":\"2022-11-28T15:45:45\",\"seller\":\"1.2.398578\",\"for_sale\":2000000,\"sell_price\":{\"base\":{\"amount\":2000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":129365483,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39819644\",\"expiration\":\"2022-11-29T03:55:12\",\"seller\":\"1.2.384119\",\"for_sale\":3000000,\"sell_price\":{\"base\":{\"amount\":3000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":195000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39711734\",\"expiration\":\"2022-11-28T15:08:31\",\"seller\":\"1.2.432748\",\"for_sale\":16620108,\"sell_price\":{\"base\":{\"amount\":17620000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":1149926586,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":0},{\"id\":\"1.7.39729717\",\"expiration\":\"2022-11-28T18:09:33\",\"seller\":\"1.2.467917\",\"for_sale\":53196,\"sell_price\":{\"base\":{\"amount\":53196,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":3485926,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39739105\",\"expiration\":\"2022-11-28T19:46:30\",\"seller\":\"1.2.450915\",\"for_sale\":1330000,\"sell_price\":{\"base\":{\"amount\":1330000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":87500000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826277\",\"expiration\":\"2022-11-29T04:27:27\",\"seller\":\"1.2.384119\",\"for_sale\":3000000,\"sell_price\":{\"base\":{\"amount\":3000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":198000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39709056\",\"expiration\":\"2022-11-28T14:33:54\",\"seller\":\"1.2.174467\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":330033003,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1221},{\"id\":\"1.7.39710371\",\"expiration\":\"2022-11-28T14:51:11\",\"seller\":\"1.2.424868\",\"for_sale\":50000000,\"sell_price\":{\"base\":{\"amount\":50000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":3309066843,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39645110\",\"expiration\":\"2022-11-28T05:57:36\",\"seller\":\"1.2.398578\",\"for_sale\":150000,\"sell_price\":{\"base\":{\"amount\":150000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":10000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39761502\",\"expiration\":\"2022-11-28T22:57:42\",\"seller\":\"1.2.101673\",\"for_sale\":43700400,\"sell_price\":{\"base\":{\"amount\":43700400,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":2913360000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39770621\",\"expiration\":\"2022-11-28T23:27:40\",\"seller\":\"1.2.445796\",\"for_sale\":11248500,\"sell_price\":{\"base\":{\"amount\":11248500,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":749900000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826381\",\"expiration\":\"2022-11-29T04:28:25\",\"seller\":\"1.2.384119\",\"for_sale\":3000000,\"sell_price\":{\"base\":{\"amount\":3000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":201000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39739797\",\"expiration\":\"2022-11-28T19:59:08\",\"seller\":\"1.2.444127\",\"for_sale\":952159,\"sell_price\":{\"base\":{\"amount\":952159,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":63839890,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39675312\",\"expiration\":\"2022-11-28T09:46:14\",\"seller\":\"1.2.398578\",\"for_sale\":740500,\"sell_price\":{\"base\":{\"amount\":740500,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":50000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39706939\",\"expiration\":\"2022-11-28T14:13:34\",\"seller\":\"1.2.443284\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":337837837,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39732387\",\"expiration\":\"2022-11-28T18:37:11\",\"seller\":\"1.2.384119\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":340000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39707139\",\"expiration\":\"2022-11-28T14:15:15\",\"seller\":\"1.2.443284\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":340136054,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39648077\",\"expiration\":\"2022-11-28T05:25:03\",\"seller\":\"1.2.161162\",\"for_sale\":4572310,\"sell_price\":{\"base\":{\"amount\":4572310,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":312529733,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39587569\",\"expiration\":\"2022-11-27T21:48:15\",\"seller\":\"1.2.449122\",\"for_sale\":2884418,\"sell_price\":{\"base\":{\"amount\":2884418,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":197553404,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39568900\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.166710\",\"for_sale\":15149,\"sell_price\":{\"base\":{\"amount\":15149,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":1039535,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39706235\",\"expiration\":\"2022-11-28T14:09:12\",\"seller\":\"1.2.443284\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":344827586,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39651001\",\"expiration\":\"2022-11-28T06:50:14\",\"seller\":\"1.2.398578\",\"for_sale\":290000,\"sell_price\":{\"base\":{\"amount\":290000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":20000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39774063\",\"expiration\":\"2022-11-28T23:40:02\",\"seller\":\"1.2.386019\",\"for_sale\":849700,\"sell_price\":{\"base\":{\"amount\":849700,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":58600000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39637259\",\"expiration\":\"2022-11-28T04:47:01\",\"seller\":\"1.2.452887\",\"for_sale\":28990,\"sell_price\":{\"base\":{\"amount\":28990,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":2000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39732395\",\"expiration\":\"2022-11-28T18:37:17\",\"seller\":\"1.2.384119\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":345000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39551534\",\"expiration\":\"2022-11-27T17:38:00\",\"seller\":\"1.2.33253\",\"for_sale\":179760,\"sell_price\":{\"base\":{\"amount\":179760,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":12441798,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39533002\",\"expiration\":\"2022-11-27T15:22:59\",\"seller\":\"1.2.388453\",\"for_sale\":933982,\"sell_price\":{\"base\":{\"amount\":2548625,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":176987851,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":0},{\"id\":\"1.7.39788915\",\"expiration\":\"1963-12-26T10:31:44\",\"seller\":\"1.2.92842\",\"for_sale\":9789859,\"sell_price\":{\"base\":{\"amount\":9789859,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":681535516,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39509337\",\"expiration\":\"2018-11-27T04:45:12\",\"seller\":\"1.2.465200\",\"for_sale\":14703,\"sell_price\":{\"base\":{\"amount\":28586,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":2000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":0},{\"id\":\"1.7.39732399\",\"expiration\":\"2022-11-28T18:37:28\",\"seller\":\"1.2.384119\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":350000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39739763\",\"expiration\":\"2022-11-28T19:58:55\",\"seller\":\"1.2.444127\",\"for_sale\":1210229,\"sell_price\":{\"base\":{\"amount\":1210229,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":85199944,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39724012\",\"expiration\":\"2022-11-28T17:07:09\",\"seller\":\"1.2.146567\",\"for_sale\":2000000,\"sell_price\":{\"base\":{\"amount\":2000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":141843971,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39574007\",\"expiration\":\"2022-11-27T19:40:39\",\"seller\":\"1.2.146567\",\"for_sale\":140901,\"sell_price\":{\"base\":{\"amount\":140901,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":10000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39732407\",\"expiration\":\"2022-11-28T18:37:34\",\"seller\":\"1.2.384119\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":355000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39588063\",\"expiration\":\"2022-11-27T21:51:29\",\"seller\":\"1.2.394864\",\"for_sale\":1500000,\"sell_price\":{\"base\":{\"amount\":1500000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":106759916,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39726395\",\"expiration\":\"2022-11-28T17:27:43\",\"seller\":\"1.2.25932\",\"for_sale\":159689,\"sell_price\":{\"base\":{\"amount\":159689,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":11365765,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1248},{\"id\":\"1.7.39713749\",\"expiration\":\"2022-11-28T15:28:20\",\"seller\":\"1.2.467917\",\"for_sale\":17410,\"sell_price\":{\"base\":{\"amount\":17410,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":1243567,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39706578\",\"expiration\":\"2022-11-28T14:11:13\",\"seller\":\"1.2.443284\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":357142857,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39816924\",\"expiration\":\"2022-11-29T03:40:12\",\"seller\":\"1.2.445451\",\"for_sale\":28000000,\"sell_price\":{\"base\":{\"amount\":28000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":2000000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39515307\",\"expiration\":\"2022-11-27T13:19:16\",\"seller\":\"1.2.168996\",\"for_sale\":1677733,\"sell_price\":{\"base\":{\"amount\":1677733,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":120000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39592929\",\"expiration\":\"2022-11-27T22:44:52\",\"seller\":\"1.2.406209\",\"for_sale\":195300,\"sell_price\":{\"base\":{\"amount\":195300,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":14000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39491714\",\"expiration\":\"2022-11-27T10:28:09\",\"seller\":\"1.2.435836\",\"for_sale\":5789525,\"sell_price\":{\"base\":{\"amount\":5789525,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":415284575,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39487775\",\"expiration\":\"2022-11-27T10:03:25\",\"seller\":\"1.2.392110\",\"for_sale\":63392638,\"sell_price\":{\"base\":{\"amount\":80000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":\"5738682718\",\"asset_id\":\"1.3.0\"}},\"deferred_fee\":0},{\"id\":\"1.7.39517335\",\"expiration\":\"2022-11-27T13:34:50\",\"seller\":\"1.2.441250\",\"for_sale\":2050000,\"sell_price\":{\"base\":{\"amount\":2050000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":147482014,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39787574\",\"expiration\":\"2022-11-29T00:56:04\",\"seller\":\"1.2.393319\",\"for_sale\":1389000,\"sell_price\":{\"base\":{\"amount\":1389000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":100000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39732410\",\"expiration\":\"2022-11-28T18:37:41\",\"seller\":\"1.2.384119\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":360000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39517481\",\"expiration\":\"2022-11-27T13:35:35\",\"seller\":\"1.2.441250\",\"for_sale\":2050000,\"sell_price\":{\"base\":{\"amount\":2050000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":148550724,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39732419\",\"expiration\":\"2022-11-28T18:37:46\",\"seller\":\"1.2.384119\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":365000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39435065\",\"expiration\":\"2022-11-27T05:09:48\",\"seller\":\"1.2.457023\",\"for_sale\":10000,\"sell_price\":{\"base\":{\"amount\":10000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":734000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39722683\",\"expiration\":\"2022-11-28T16:55:10\",\"seller\":\"1.2.203202\",\"for_sale\":190965,\"sell_price\":{\"base\":{\"amount\":190965,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":14016831,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39484129\",\"expiration\":\"2022-11-27T09:45:08\",\"seller\":\"1.2.28914\",\"for_sale\":6805000,\"sell_price\":{\"base\":{\"amount\":6805000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":500000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39418777\",\"expiration\":\"2022-11-27T03:18:07\",\"seller\":\"1.2.23586\",\"for_sale\":4000000,\"sell_price\":{\"base\":{\"amount\":4000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":294000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39726984\",\"expiration\":\"2022-11-28T17:33:46\",\"seller\":\"1.2.33175\",\"for_sale\":7048821,\"sell_price\":{\"base\":{\"amount\":7048821,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":518299472,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39649090\",\"expiration\":\"2022-11-28T06:31:01\",\"seller\":\"1.2.444127\",\"for_sale\":1176408,\"sell_price\":{\"base\":{\"amount\":1176408,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":86959941,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39732428\",\"expiration\":\"2022-11-28T18:37:52\",\"seller\":\"1.2.384119\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":370000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39728795\",\"expiration\":\"2022-11-28T17:56:51\",\"seller\":\"1.2.430356\",\"for_sale\":10302063,\"sell_price\":{\"base\":{\"amount\":10302063,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":763115777,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1236},{\"id\":\"1.7.39512722\",\"expiration\":\"2022-11-27T13:04:24\",\"seller\":\"1.2.169756\",\"for_sale\":4995000,\"sell_price\":{\"base\":{\"amount\":4995000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":370000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39418900\",\"expiration\":\"2022-11-27T03:18:58\",\"seller\":\"1.2.23586\",\"for_sale\":3270000,\"sell_price\":{\"base\":{\"amount\":3270000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":245250000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39497263\",\"expiration\":\"2022-11-27T11:11:47\",\"seller\":\"1.2.363509\",\"for_sale\":85806,\"sell_price\":{\"base\":{\"amount\":85806,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":6435450,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39732437\",\"expiration\":\"2022-11-28T18:37:58\",\"seller\":\"1.2.384119\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":375000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39509513\",\"expiration\":\"2022-11-27T12:47:47\",\"seller\":\"1.2.152063\",\"for_sale\":1055407,\"sell_price\":{\"base\":{\"amount\":1055407,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":79353909,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39592860\",\"expiration\":\"2022-11-27T22:43:27\",\"seller\":\"1.2.406209\",\"for_sale\":98420,\"sell_price\":{\"base\":{\"amount\":98420,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":7400000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39732443\",\"expiration\":\"2022-11-28T18:38:03\",\"seller\":\"1.2.384119\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":380000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39697357\",\"expiration\":\"2022-11-28T13:01:34\",\"seller\":\"1.2.474330\",\"for_sale\":393000,\"sell_price\":{\"base\":{\"amount\":393000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":30000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39169827\",\"expiration\":\"2022-11-25T21:52:00\",\"seller\":\"1.2.146567\",\"for_sale\":13090,\"sell_price\":{\"base\":{\"amount\":13090,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":1000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39186397\",\"expiration\":\"2022-11-25T23:51:05\",\"seller\":\"1.2.470440\",\"for_sale\":1308010,\"sell_price\":{\"base\":{\"amount\":1308010,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":100000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39237988\",\"expiration\":\"2022-11-26T05:55:59\",\"seller\":\"1.2.394864\",\"for_sale\":956021,\"sell_price\":{\"base\":{\"amount\":956021,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":73298499,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38477978\",\"expiration\":\"2022-11-22T11:09:49\",\"seller\":\"1.2.32432\",\"for_sale\":928572,\"sell_price\":{\"base\":{\"amount\":928572,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":71428571,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38324090\",\"expiration\":\"2022-11-21T21:13:54\",\"seller\":\"1.2.34866\",\"for_sale\":2363637,\"sell_price\":{\"base\":{\"amount\":2363637,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":181818181,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38323860\",\"expiration\":\"2022-11-21T21:11:28\",\"seller\":\"1.2.110360\",\"for_sale\":100000,\"sell_price\":{\"base\":{\"amount\":100000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":7692307,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38582244\",\"expiration\":\"2022-11-22T21:36:15\",\"seller\":\"1.2.307732\",\"for_sale\":975000,\"sell_price\":{\"base\":{\"amount\":975000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":75000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39534038\",\"expiration\":\"2022-11-27T15:32:58\",\"seller\":\"1.2.89816\",\"for_sale\":26000000,\"sell_price\":{\"base\":{\"amount\":26000000,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":2000000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39419038\",\"expiration\":\"2022-11-27T03:19:36\",\"seller\":\"1.2.23586\",\"for_sale\":3624500,\"sell_price\":{\"base\":{\"amount\":3624500,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":279086500,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38309073\",\"expiration\":\"2022-11-21T19:37:30\",\"seller\":\"1.2.165359\",\"for_sale\":518520,\"sell_price\":{\"base\":{\"amount\":518520,\"asset_id\":\"1.3.121\"},\"quote\":{\"amount\":40000000,\"asset_id\":\"1.3.0\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826987\",\"expiration\":\"2022-11-28T04:31:28\",\"seller\":\"1.2.126225\",\"for_sale\":5637462,\"sell_price\":{\"base\":{\"amount\":5837574,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":100000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":0},{\"id\":\"1.7.39826962\",\"expiration\":\"2022-11-28T04:31:25\",\"seller\":\"1.2.116747\",\"for_sale\":29187564,\"sell_price\":{\"base\":{\"amount\":29187564,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":499999,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826504\",\"expiration\":\"2022-11-29T04:29:06\",\"seller\":\"1.2.470016\",\"for_sale\":4423746,\"sell_price\":{\"base\":{\"amount\":4423746,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":75782,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826958\",\"expiration\":\"1963-11-25T23:31:44\",\"seller\":\"1.2.442895\",\"for_sale\":151718210,\"sell_price\":{\"base\":{\"amount\":151718210,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":2600566,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39825989\",\"expiration\":\"1963-11-25T17:31:44\",\"seller\":\"1.2.359372\",\"for_sale\":70741090,\"sell_price\":{\"base\":{\"amount\":70741090,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1214423,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826801\",\"expiration\":\"1963-11-25T14:31:44\",\"seller\":\"1.2.475168\",\"for_sale\":44187474,\"sell_price\":{\"base\":{\"amount\":44187474,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":758636,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39819539\",\"expiration\":\"2022-11-29T03:54:38\",\"seller\":\"1.2.456209\",\"for_sale\":8799002,\"sell_price\":{\"base\":{\"amount\":8799002,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":151078,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39817429\",\"expiration\":\"2022-11-29T03:40:58\",\"seller\":\"1.2.384257\",\"for_sale\":146100000,\"sell_price\":{\"base\":{\"amount\":146100000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":2512890,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39817124\",\"expiration\":\"2022-11-29T03:41:13\",\"seller\":\"1.2.456209\",\"for_sale\":8799002,\"sell_price\":{\"base\":{\"amount\":8799002,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":151342,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39827040\",\"expiration\":\"2017-11-29T05:31:31\",\"seller\":\"1.2.359723\",\"for_sale\":59168823,\"sell_price\":{\"base\":{\"amount\":59168823,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1021695,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826066\",\"expiration\":\"1963-11-25T22:31:44\",\"seller\":\"1.2.475521\",\"for_sale\":262398871,\"sell_price\":{\"base\":{\"amount\":262398871,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":4531618,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39814186\",\"expiration\":\"2022-11-29T03:30:39\",\"seller\":\"1.2.384257\",\"for_sale\":28800000,\"sell_price\":{\"base\":{\"amount\":28800000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":498237,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826021\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.403430\",\"for_sale\":1768527,\"sell_price\":{\"base\":{\"amount\":1768527,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":30602,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39825992\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.395549\",\"for_sale\":884263,\"sell_price\":{\"base\":{\"amount\":884263,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":15301,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826008\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.368699\",\"for_sale\":884263,\"sell_price\":{\"base\":{\"amount\":884263,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":15301,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826013\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.354462\",\"for_sale\":884263,\"sell_price\":{\"base\":{\"amount\":884263,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":15301,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39825997\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.443891\",\"for_sale\":8842636,\"sell_price\":{\"base\":{\"amount\":8842636,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":153014,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826005\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.423992\",\"for_sale\":8842636,\"sell_price\":{\"base\":{\"amount\":8842636,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":153014,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826034\",\"expiration\":\"1963-11-25T22:31:44\",\"seller\":\"1.2.95758\",\"for_sale\":88426363,\"sell_price\":{\"base\":{\"amount\":88426363,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1530149,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826012\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.465862\",\"for_sale\":104343108,\"sell_price\":{\"base\":{\"amount\":104343108,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1805576,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39813053\",\"expiration\":\"2022-11-29T03:26:57\",\"seller\":\"1.2.440027\",\"for_sale\":793249585,\"sell_price\":{\"base\":{\"amount\":793249585,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":13739082,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826041\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.265788\",\"for_sale\":1716847,\"sell_price\":{\"base\":{\"amount\":1716847,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":29855,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39812537\",\"expiration\":\"2022-11-29T03:23:59\",\"seller\":\"1.2.456209\",\"for_sale\":4580116,\"sell_price\":{\"base\":{\"amount\":4580116,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":79694,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39808094\",\"expiration\":\"2022-11-29T02:58:27\",\"seller\":\"1.2.384257\",\"for_sale\":65885582,\"sell_price\":{\"base\":{\"amount\":237000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":4130886,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":0},{\"id\":\"1.7.39825788\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.398049\",\"for_sale\":29415141,\"sell_price\":{\"base\":{\"amount\":29415141,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":512760,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39825974\",\"expiration\":\"1963-11-26T01:31:44\",\"seller\":\"1.2.284457\",\"for_sale\":176852726,\"sell_price\":{\"base\":{\"amount\":176852726,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":3090599,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826017\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.410471\",\"for_sale\":265279089,\"sell_price\":{\"base\":{\"amount\":265279089,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":4635899,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826241\",\"expiration\":\"2017-12-06T04:27:13\",\"seller\":\"1.2.171023\",\"for_sale\":881150,\"sell_price\":{\"base\":{\"amount\":881150,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":15429,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39815120\",\"expiration\":\"2022-11-29T03:34:31\",\"seller\":\"1.2.96660\",\"for_sale\":\"57100000000\",\"sell_price\":{\"base\":{\"amount\":\"57100000000\",\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1000000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39824425\",\"expiration\":\"2017-12-06T04:19:15\",\"seller\":\"1.2.287187\",\"for_sale\":15907571,\"sell_price\":{\"base\":{\"amount\":15907571,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":279521,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826399\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.399496\",\"for_sale\":448610,\"sell_price\":{\"base\":{\"amount\":448610,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":7908,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39825790\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.396680\",\"for_sale\":8833375,\"sell_price\":{\"base\":{\"amount\":8833375,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":156044,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826308\",\"expiration\":\"1963-11-29T12:38:45\",\"seller\":\"1.2.36449\",\"for_sale\":\"4421486581\",\"sell_price\":{\"base\":{\"amount\":\"4421486581\",\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":78781360,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826064\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.399578\",\"for_sale\":2652790,\"sell_price\":{\"base\":{\"amount\":2652790,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":47267,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826003\",\"expiration\":\"1963-11-25T15:31:44\",\"seller\":\"1.2.453726\",\"for_sale\":2521441,\"sell_price\":{\"base\":{\"amount\":2521441,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":44927,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826816\",\"expiration\":\"1963-11-29T12:38:45\",\"seller\":\"1.2.33015\",\"for_sale\":7356955,\"sell_price\":{\"base\":{\"amount\":7356955,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":131374,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38751649\",\"expiration\":\"2022-11-23T17:29:56\",\"seller\":\"1.2.203202\",\"for_sale\":112000000,\"sell_price\":{\"base\":{\"amount\":112000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":2000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39783095\",\"expiration\":\"2022-11-29T00:21:53\",\"seller\":\"1.2.437844\",\"for_sale\":161968793,\"sell_price\":{\"base\":{\"amount\":161968793,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":2899241,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826049\",\"expiration\":\"1963-11-25T14:31:44\",\"seller\":\"1.2.381972\",\"for_sale\":26527908,\"sell_price\":{\"base\":{\"amount\":26527908,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":474952,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39718235\",\"expiration\":\"2022-11-28T16:13:19\",\"seller\":\"1.2.99949\",\"for_sale\":\"6584400000\",\"sell_price\":{\"base\":{\"amount\":\"6584400000\",\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":118024711,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826039\",\"expiration\":\"1963-11-25T14:31:44\",\"seller\":\"1.2.414841\",\"for_sale\":44213181,\"sell_price\":{\"base\":{\"amount\":44213181,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":794617,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39825977\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.4890\",\"for_sale\":858029,\"sell_price\":{\"base\":{\"amount\":858029,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":15435,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.25560191\",\"expiration\":\"2022-09-01T16:47:07\",\"seller\":\"1.2.338201\",\"for_sale\":3494185,\"sell_price\":{\"base\":{\"amount\":3494185,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":62895,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38522042\",\"expiration\":\"2022-11-22T14:49:29\",\"seller\":\"1.2.183905\",\"for_sale\":2222223,\"sell_price\":{\"base\":{\"amount\":2222223,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":40000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.23679244\",\"expiration\":\"2022-08-18T14:23:42\",\"seller\":\"1.2.32432\",\"for_sale\":55555556,\"sell_price\":{\"base\":{\"amount\":55555556,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38835207\",\"expiration\":\"2022-11-24T06:11:33\",\"seller\":\"1.2.26041\",\"for_sale\":555555556,\"sell_price\":{\"base\":{\"amount\":555555556,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":10000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.36775910\",\"expiration\":\"2022-11-14T06:54:06\",\"seller\":\"1.2.461373\",\"for_sale\":14000000,\"sell_price\":{\"base\":{\"amount\":14000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":252000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38343747\",\"expiration\":\"2022-11-21T23:43:04\",\"seller\":\"1.2.114583\",\"for_sale\":500000000,\"sell_price\":{\"base\":{\"amount\":500000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":9000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826000\",\"expiration\":\"1963-11-25T14:31:44\",\"seller\":\"1.2.379967\",\"for_sale\":8381645,\"sell_price\":{\"base\":{\"amount\":8381645,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":151499,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38446613\",\"expiration\":\"2022-11-22T08:19:40\",\"seller\":\"1.2.444127\",\"for_sale\":77580077,\"sell_price\":{\"base\":{\"amount\":77580077,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1406451,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39826878\",\"expiration\":\"1963-11-25T22:31:44\",\"seller\":\"1.2.450176\",\"for_sale\":4013160,\"sell_price\":{\"base\":{\"amount\":4013160,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":72924,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39813365\",\"expiration\":\"2022-11-29T03:28:19\",\"seller\":\"1.2.470558\",\"for_sale\":27500000,\"sell_price\":{\"base\":{\"amount\":27500000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":500000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39822798\",\"expiration\":\"1963-11-25T09:31:44\",\"seller\":\"1.2.165710\",\"for_sale\":4248765,\"sell_price\":{\"base\":{\"amount\":4248765,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":77287,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38303494\",\"expiration\":\"2022-11-21T18:26:05\",\"seller\":\"1.2.457225\",\"for_sale\":64196743,\"sell_price\":{\"base\":{\"amount\":64196743,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1179750,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38242275\",\"expiration\":\"2022-11-21T11:26:50\",\"seller\":\"1.2.430159\",\"for_sale\":4000000,\"sell_price\":{\"base\":{\"amount\":4000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":73600,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38550329\",\"expiration\":\"2022-11-22T17:20:51\",\"seller\":\"1.2.422975\",\"for_sale\":51173959,\"sell_price\":{\"base\":{\"amount\":51173959,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":946718,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.25546186\",\"expiration\":\"2022-09-01T14:53:32\",\"seller\":\"1.2.158537\",\"for_sale\":212001462,\"sell_price\":{\"base\":{\"amount\":212001462,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":3962986,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39617047\",\"expiration\":\"2022-11-28T01:53:19\",\"seller\":\"1.2.159614\",\"for_sale\":159489634,\"sell_price\":{\"base\":{\"amount\":159489634,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":3000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.15353005\",\"expiration\":\"2022-07-09T12:07:25\",\"seller\":\"1.2.267\",\"for_sale\":265789474,\"sell_price\":{\"base\":{\"amount\":265789474,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":5050000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.21991367\",\"expiration\":\"2022-08-07T07:17:36\",\"seller\":\"1.2.32432\",\"for_sale\":52631579,\"sell_price\":{\"base\":{\"amount\":52631579,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38839709\",\"expiration\":\"2022-11-24T06:37:46\",\"seller\":\"1.2.26041\",\"for_sale\":526315790,\"sell_price\":{\"base\":{\"amount\":526315790,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":10000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39771147\",\"expiration\":\"2022-11-28T23:28:53\",\"seller\":\"1.2.375610\",\"for_sale\":100000000,\"sell_price\":{\"base\":{\"amount\":100000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1900000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38446699\",\"expiration\":\"2022-11-22T08:19:48\",\"seller\":\"1.2.444127\",\"for_sale\":76266740,\"sell_price\":{\"base\":{\"amount\":76266740,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1451774,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.18636813\",\"expiration\":\"2022-07-21T04:14:18\",\"seller\":\"1.2.116476\",\"for_sale\":525000000,\"sell_price\":{\"base\":{\"amount\":525000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":10000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.37077687\",\"expiration\":\"2022-11-16T01:37:47\",\"seller\":\"1.2.203594\",\"for_sale\":300000000,\"sell_price\":{\"base\":{\"amount\":300000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":5752293,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38303501\",\"expiration\":\"2022-11-21T18:26:13\",\"seller\":\"1.2.457225\",\"for_sale\":64196743,\"sell_price\":{\"base\":{\"amount\":64196743,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1243947,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.14883703\",\"expiration\":\"2022-07-07T22:48:58\",\"seller\":\"1.2.267\",\"for_sale\":258974359,\"sell_price\":{\"base\":{\"amount\":258974359,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":5050000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38333659\",\"expiration\":\"2022-11-21T22:24:06\",\"seller\":\"1.2.443514\",\"for_sale\":500000000,\"sell_price\":{\"base\":{\"amount\":500000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":9750000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.23484383\",\"expiration\":\"2022-08-17T06:57:34\",\"seller\":\"1.2.266278\",\"for_sale\":51000000,\"sell_price\":{\"base\":{\"amount\":51000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38258389\",\"expiration\":\"2022-11-21T12:48:52\",\"seller\":\"1.2.388463\",\"for_sale\":199000000,\"sell_price\":{\"base\":{\"amount\":199000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":3940200,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39534012\",\"expiration\":\"2022-11-27T15:32:14\",\"seller\":\"1.2.203202\",\"for_sale\":199900000,\"sell_price\":{\"base\":{\"amount\":199900000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":3958020,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39535581\",\"expiration\":\"2022-11-27T15:44:53\",\"seller\":\"1.2.203202\",\"for_sale\":199900000,\"sell_price\":{\"base\":{\"amount\":199900000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":3958020,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39617049\",\"expiration\":\"2022-11-28T01:53:26\",\"seller\":\"1.2.159614\",\"for_sale\":151438668,\"sell_price\":{\"base\":{\"amount\":151438668,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":3000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38333697\",\"expiration\":\"2022-11-21T22:24:19\",\"seller\":\"1.2.443514\",\"for_sale\":500000000,\"sell_price\":{\"base\":{\"amount\":500000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":9950000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39516285\",\"expiration\":\"2022-11-27T13:27:48\",\"seller\":\"1.2.36352\",\"for_sale\":753012049,\"sell_price\":{\"base\":{\"amount\":753012049,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":15000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39799318\",\"expiration\":\"2022-11-29T01:58:14\",\"seller\":\"1.2.398578\",\"for_sale\":44016437,\"sell_price\":{\"base\":{\"amount\":44016437,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":879448,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38446740\",\"expiration\":\"2022-11-22T08:19:58\",\"seller\":\"1.2.444127\",\"for_sale\":75015943,\"sell_price\":{\"base\":{\"amount\":75015943,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1499362,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39563988\",\"expiration\":\"2022-11-27T19:11:12\",\"seller\":\"1.2.155012\",\"for_sale\":\"10000000000\",\"sell_price\":{\"base\":{\"amount\":\"10000000000\",\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":199990000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38502824\",\"expiration\":\"2022-11-22T13:39:08\",\"seller\":\"1.2.422975\",\"for_sale\":33949422,\"sell_price\":{\"base\":{\"amount\":33949422,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":678988,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.26162941\",\"expiration\":\"2022-09-05T04:26:53\",\"seller\":\"1.2.114303\",\"for_sale\":87304970,\"sell_price\":{\"base\":{\"amount\":87304970,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1746099,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.30738611\",\"expiration\":\"2022-10-04T20:20:55\",\"seller\":\"1.2.394462\",\"for_sale\":648063095,\"sell_price\":{\"base\":{\"amount\":648063095,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":12961261,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39489300\",\"expiration\":\"2022-11-27T10:12:44\",\"seller\":\"1.2.437837\",\"for_sale\":548553862,\"sell_price\":{\"base\":{\"amount\":548553862,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":10971077,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.14756774\",\"expiration\":\"2022-07-07T12:10:46\",\"seller\":\"1.2.267\",\"for_sale\":252500000,\"sell_price\":{\"base\":{\"amount\":252500000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":5050000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.18636812\",\"expiration\":\"2022-07-21T04:14:07\",\"seller\":\"1.2.116476\",\"for_sale\":500000000,\"sell_price\":{\"base\":{\"amount\":500000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":10000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.21991369\",\"expiration\":\"2022-08-07T07:17:44\",\"seller\":\"1.2.32432\",\"for_sale\":50000000,\"sell_price\":{\"base\":{\"amount\":50000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.23216503\",\"expiration\":\"2022-08-15T08:57:18\",\"seller\":\"1.2.266278\",\"for_sale\":50000000,\"sell_price\":{\"base\":{\"amount\":50000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":1000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.23818382\",\"expiration\":\"2022-08-19T14:09:22\",\"seller\":\"1.2.266278\",\"for_sale\":25000000,\"sell_price\":{\"base\":{\"amount\":25000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":500000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.33199477\",\"expiration\":\"2022-10-23T08:11:42\",\"seller\":\"1.2.367064\",\"for_sale\":5000000,\"sell_price\":{\"base\":{\"amount\":5000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":100000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.37868756\",\"expiration\":\"2022-11-19T23:13:33\",\"seller\":\"1.2.104650\",\"for_sale\":\"4500000000\",\"sell_price\":{\"base\":{\"amount\":\"4500000000\",\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":90000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38050853\",\"expiration\":\"2022-11-20T17:59:01\",\"seller\":\"1.2.32653\",\"for_sale\":185000000,\"sell_price\":{\"base\":{\"amount\":185000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":3700000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38270285\",\"expiration\":\"2022-11-21T14:08:27\",\"seller\":\"1.2.404345\",\"for_sale\":2500000,\"sell_price\":{\"base\":{\"amount\":2500000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":50000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.39051789\",\"expiration\":\"2022-11-25T05:52:17\",\"seller\":\"1.2.165313\",\"for_sale\":250000000,\"sell_price\":{\"base\":{\"amount\":250000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":5000000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38038751\",\"expiration\":\"2022-11-20T17:09:43\",\"seller\":\"1.2.99949\",\"for_sale\":\"10628496871\",\"sell_price\":{\"base\":{\"amount\":\"10628496871\",\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":214729924,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38333708\",\"expiration\":\"2022-11-21T22:24:36\",\"seller\":\"1.2.443514\",\"for_sale\":500000000,\"sell_price\":{\"base\":{\"amount\":500000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":10150000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.37016152\",\"expiration\":\"2022-11-15T17:10:45\",\"seller\":\"1.2.99896\",\"for_sale\":1000000000,\"sell_price\":{\"base\":{\"amount\":1000000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":20300010,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38294306\",\"expiration\":\"2022-11-21T17:09:33\",\"seller\":\"1.2.99949\",\"for_sale\":\"9593198903\",\"sell_price\":{\"base\":{\"amount\":\"9593198903\",\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":195599090,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38038797\",\"expiration\":\"2022-11-20T17:09:56\",\"seller\":\"1.2.99949\",\"for_sale\":\"10628496871\",\"sell_price\":{\"base\":{\"amount\":\"10628496871\",\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":216855623,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.14673195\",\"expiration\":\"2022-07-07T03:21:11\",\"seller\":\"1.2.267\",\"for_sale\":183076053,\"sell_price\":{\"base\":{\"amount\":246341464,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":5050000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":0},{\"id\":\"1.7.39707664\",\"expiration\":\"2022-11-28T14:20:03\",\"seller\":\"1.2.443284\",\"for_sale\":3096209579,\"sell_price\":{\"base\":{\"amount\":3096209579,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":63695223,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213},{\"id\":\"1.7.38333740\",\"expiration\":\"2022-11-21T22:24:48\",\"seller\":\"1.2.443514\",\"for_sale\":500000000,\"sell_price\":{\"base\":{\"amount\":500000000,\"asset_id\":\"1.3.0\"},\"quote\":{\"amount\":10300000,\"asset_id\":\"1.3.121\"}},\"deferred_fee\":1213}]}";
        GsonBuilder builder = new GsonBuilder();
        builder.registerTypeAdapter(AssetAmount.class, new AssetAmount.AssetAmountDeserializer());
        builder.registerTypeAdapter(UserAccount.class, new UserAccount.UserAccountSimpleDeserializer());
        builder.registerTypeAdapter(LimitOrder.class, new LimitOrder.LimitOrderDeserializer());
        Type GetLimitOrdersResponse = new TypeToken<WitnessResponse<List<LimitOrder>>>() {}.getType();
        WitnessResponse<List<LimitOrder>> witnessResponse = builder.create().fromJson(serializedOrderBook, GetLimitOrdersResponse);
        List<LimitOrder> orders = witnessResponse.result;
        OrderBook orderBook = new OrderBook(orders);

        // Testing obtained quote calculation
        long receivedValue = 56111;
        long agoriseCut = 280;
        UnsignedLong merchantsQuote = orderBook.calculateObtainedQuote(new AssetAmount(UnsignedLong.valueOf(receivedValue - agoriseCut), new Asset("1.3.0")));
        Assert.assertEquals(941, merchantsQuote.longValue());

        // Testing required base calculation
        UnsignedLong baseAmount = orderBook.calculateRequiredBase(new AssetAmount(UnsignedLong.valueOf(10000), new Asset("1.3.121")));
        Assert.assertEquals(593000, baseAmount.longValue());
    }
}
